@@ -20,13 +20,12 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include "gailrange.h"
-#include "gailrangefactory.h"
 #include "gailadjustment.h"
 
 static void	    gail_range_class_init        (GailRangeClass *klass);
 
-static void         gail_range_real_init         (GailWidget     *widget,
-                                                  GtkWidget      *gtk_widget);
+static void         gail_range_real_initialize   (AtkObject      *obj,
+                                                  gpointer      data);
 
 static void         gail_range_finalize          (GObject        *object);
 static AtkStateSet* gail_range_ref_state_set     (AtkObject      *obj);
@@ -48,7 +47,6 @@ static void         gail_range_value_changed     (GtkAdjustment  *adjustment,
                                                   gpointer       data);
 static GailWidgetClass *parent_class = NULL;
 
-
 GType
 gail_range_get_type (void)
 {
@@ -96,13 +94,13 @@ gail_range_class_init		(GailRangeClass *klass)
   widget_class = (GailWidgetClass*)klass;
 
   widget_class->notify_gtk = gail_range_real_notify_gtk;
-  widget_class->init = gail_range_real_init;
 
   class->ref_state_set = gail_range_ref_state_set;
+  class->initialize = gail_range_real_initialize;
 
   gobject_class->finalize = gail_range_finalize;
 
-  parent_class = g_type_class_ref (GAIL_TYPE_WIDGET);
+  parent_class = g_type_class_peek_parent (klass);
 }
 
 AtkObject* 
@@ -115,9 +113,8 @@ gail_range_new (GtkWidget *widget)
 
   object = g_object_new (GAIL_TYPE_RANGE, NULL);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
   accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
 
   /*
    * Assumed to GtkScale (either GtkHScale or GtkVScale)
@@ -128,15 +125,15 @@ gail_range_new (GtkWidget *widget)
 }
 
 static void
-gail_range_real_init (GailWidget         *widget,
-                      GtkWidget          *gtk_widget)
+gail_range_real_initialize (AtkObject *obj,
+                            gpointer  data)
 {
-  GailRange *range = GAIL_RANGE (widget);
+  GailRange *range = GAIL_RANGE (obj);
   GtkRange *gtk_range;
 
-  GAIL_WIDGET_CLASS (parent_class)->init (widget, gtk_widget);
+  ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
-  gtk_range = GTK_RANGE (gtk_widget);
+  gtk_range = GTK_RANGE (data);
   /*
    * If a GtkAdjustment already exists for the GtkRange,
    * create the GailAdjustment
@@ -147,7 +144,7 @@ gail_range_real_init (GailWidget         *widget,
       g_signal_connect (gtk_range->adjustment,
                         "value-changed",
                         G_CALLBACK (gail_range_value_changed),
-                        widget);
+                        range);
     }
   else
     range->adjustment = NULL;
@@ -249,18 +246,27 @@ gail_range_get_minimum_value (AtkValue		*obj,
 static gboolean	 gail_range_set_current_value (AtkValue		*obj,
                                                const GValue	*value)
 {
-  GailRange *range;
+  GtkWidget *widget;
 
   g_return_val_if_fail (GAIL_IS_RANGE (obj), FALSE);
 
-  range = GAIL_RANGE (obj);
-  if (range->adjustment == NULL)
-    /*
-     * Adjustment has not been specified
-     */
+  widget = GTK_ACCESSIBLE (obj)->widget;
+  if (widget == NULL)
     return FALSE;
 
-  return atk_value_set_current_value (ATK_VALUE (range->adjustment), value);
+  if (G_VALUE_HOLDS_DOUBLE (value))
+    {
+      GtkRange *range = GTK_RANGE (widget);
+      gdouble new_value;
+
+      new_value = g_value_get_double (value);
+      gtk_range_set_value (range, new_value);
+      return TRUE;
+    }
+  else
+    {
+      return FALSE;
+    }
 }
 
 static void
@@ -270,6 +276,14 @@ gail_range_finalize (GObject            *object)
 
   if (range->adjustment)
     {
+      /*
+       * The GtkAdjustment may live on so we need to dicsonnect the
+       * signal handler
+       */
+      g_signal_handlers_disconnect_by_func (GAIL_ADJUSTMENT (range->adjustment)->adjustment,
+                                            gail_range_value_changed,
+                                            range);
+                                           
       g_object_unref (range->adjustment);
       range->adjustment = NULL;
     }
